/*

	Color MDEF
	by Jordan Zimmerman
	(c)1995 All rights reserved worldwide.

	Unlimited use is hereby granted without restriction.  However,
	the author would appreciate credit if possible.  Please send
	comments, questions, bugs, etc. to jordanz@altura.com
	
	This code implements some utilities to convert between RGBColors
	and their string representations that go into the Menu Item text.
	
	Change History:
	
	1.0.0	10/05/95	JLZ		Created inital version

*/

#include "color_mdef_utils.h"

static void str_to_component(const unsigned char* ptr, unsigned short& component);
static void component_to_str(unsigned short component, unsigned char* str);

//~~~~~~~~~~~~~~Global Functions

// set the given menu item to the given color
void CMSetMenuItemColor(MenuHandle menu_h, short item_number, const RGBColor& color)
{

	Str255		color_pstr;
	::CMColorToPstr(color, color_pstr);
	
	::SetMenuItemText(menu_h, item_number, color_pstr);

} // CMSetMenuItemColor

// return the color for the given menu item
void CMGetMenuItemColor(MenuHandle menu_h, short item_number, RGBColor& color)
{

	Str255		color_pstr;
	::GetMenuItemText(menu_h, item_number, color_pstr);
	
	::CMPstrToColor(color_pstr, color);

} // CMGetMenuItemColor

// convert the given formatted 12 byte pstring into an RGBColor
// the pstring should be in the form RRRRGGGGBBBB where
// RRRR, GGGG, and BBBB are each ASCII four byte hex numbers
void CMPstrToColor(ConstStr255Param pstr, RGBColor& color)
{

	::str_to_component(pstr + 1, color.red);
	::str_to_component(pstr + 5, color.green);
	::str_to_component(pstr + 9, color.blue);

} // CMPstrToColor

// convert the given RGBColor into a 12 byte formatted pstring
void CMColorToPstr(const RGBColor& color, StringPtr pstr)
{
	
	pstr[0] = 12;
	
	::component_to_str(color.red, pstr + 1);
	::component_to_str(color.green, pstr + 5);
	::component_to_str(color.blue, pstr + 9);
	
} // CMColorToPstr

//~~~~~~~~~~~~~~Local Functions

inline unsigned short hex_to_num(unsigned short hex_char)
{

	return (hex_char < 'A') ? (hex_char - '0') : (hex_char - 'A' + 10);

} // hex_to_num

static void str_to_component(const unsigned char* ptr, unsigned short& component)
{

	component = hex_to_num(ptr[3]);
	component += hex_to_num(ptr[2]) * 16;
	component += hex_to_num(ptr[1]) * 256;
	component += hex_to_num(ptr[0]) * 4096;

} // str_to_component

inline unsigned char num_to_hex(unsigned char num)
{

	return (num < 10) ? (num + '0') : (num - 10 + 'A');

} // num_to_hex

static void component_to_str(unsigned short component, unsigned char* str)
{

	str[0] = num_to_hex((component >> 12) & 0x0f);
	str[1] = num_to_hex((component >> 8) & 0x0f);
	str[2] = num_to_hex((component >> 4) & 0x0f);
	str[3] = num_to_hex(component & 0x0F);

} // component_to_str
