#include "color_mdef_utils.h"

enum
{

	file_menu_id	= 1,
		quit_command 	= 1,
		
	color_menu_id	= 2,
	
	popup_menu_id	= 3

};

static void add_colors(MenuHandle menu_h);
static void handle_menu(long menu_code);
static void handle_color_menu(short item_number);
static void do_popup(Point global_pt);

static int			app_is_done = FALSE;
static RGBColor		current_color = {0xffff, 0xffff, 0xffff};
static WindowPtr	window_ptr = NULL;

void main(void)
{

	InitGraf(&qd.thePort);
	InitFonts();
	FlushEvents(everyEvent, 0);
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);
	InitCursor();
	
	// insert the menus
	MenuHandle		menu_h = ::GetMenu(file_menu_id);
	if ( !menu_h )
	{
		return;
	}
	::InsertMenu(menu_h, 0);
	menu_h = ::GetMenu(color_menu_id);
	if ( !menu_h )
	{
		return;
	}
	::InsertMenu(menu_h, 0);
	::add_colors(menu_h);
	
	::DrawMenuBar();
	
	Rect		window_bounds_r = {50, 50, 200, 350};
	// a window to show the color
	window_ptr = ::NewCWindow(NULL, &window_bounds_r, "\pCurrent Color", true, noGrowDocProc, (WindowPtr)-1, false, 0);
	if ( !window_ptr )
	{
		return;
	}
	::SetPort(window_ptr);
	
	// standard Mac Event loop
	while ( !app_is_done )
	{
		EventRecord		the_event;
		
		::SystemTask();	// I forget if this is still needed these days
		::WaitNextEvent(everyEvent, &the_event, 1000, NULL);
		
		switch ( the_event.what )
		{
			case updateEvt:
			{
				::BeginUpdate(window_ptr);

				::RGBForeColor(&current_color);
				::PaintRect(&window_ptr->portRect);
				
				::ForeColor(blackColor);
				::MoveTo(10, 16);
				::DrawString("\pColor MDEF");
				::MoveTo(10, 32);
				::DrawString("\pby Jordan Zimmerman (jordanz@altura.com)");
				::MoveTo(10, 64);
				::DrawString("\pClick anywhere in this window");
				::MoveTo(10, 80);
				::DrawString("\pto do a popup menu.");
				
				::EndUpdate(window_ptr);
				break;
			}
		
			case keyDown:
			{
				if ( the_event.modifiers & cmdKey )
				{
					::handle_menu(::MenuKey(the_event.message & charCodeMask));
					::HiliteMenu(0);
				}
				break;
			}
		
			case mouseDown:
			{
				WindowPtr		found_window_ptr;
				switch ( ::FindWindow(the_event.where, &found_window_ptr) )
				{
					case inMenuBar:
					{
						::handle_menu(::MenuSelect(the_event.where));
						::HiliteMenu(0);
						break;
					}
					
					case inDrag:
					{
						::DragWindow(found_window_ptr, the_event.where, &qd.screenBits.bounds);
						break;
					}
					
					case inContent:
					{
						::do_popup(the_event.where);
						break;
					}
				}
				break;
			}
		}
	}

} // main

// use the standard 256 color byte to add colors to the menu
static void add_colors(MenuHandle menu_h)
{

	// MPTA says use the bit depth + 64 to get the default color table
	CTabHandle		color_table_h = ::GetCTable(8 + 64);
	if ( !color_table_h )
	{
		return;
	}
	char		save_state = ::HGetState((Handle)color_table_h);
	::HNoPurge((Handle)color_table_h);
	
	short			color_qty = (**color_table_h).ctSize + 1;
	for ( short i = 1; i <= color_qty; ++i )
	{
		// append with a neutral string to avoid meta-character issues
		::AppendMenu(menu_h, "\p-");

		RGBColor		this_color = (**color_table_h).ctTable[i - 1].rgb;
		::CMSetMenuItemColor(menu_h, i, this_color);
	}
	
	::HSetState((Handle)color_table_h, save_state);

} // add_colors

// handle a menu selection
static void handle_menu(long menu_code)
{

	if ( HiWord(menu_code) == file_menu_id )
	{
		app_is_done = true;
	}
	else if ( HiWord(menu_code) == color_menu_id )
	{
		handle_color_menu(LoWord(menu_code));
	}

} // handle_menu

// handle a selection on the color menu
static void handle_color_menu(short item_number)
{

	MenuHandle		menu_h = ::GetMHandle(color_menu_id);
	if ( !menu_h )
	{
		return;
	}
	
	// mark the new one, turn off the mark for the others
	short			item_qty = ::CountMItems(menu_h);
	for ( short i = 1; i <= item_qty; ++i )
	{
		::SetItemMark(menu_h, i, (i == item_number) ? 1 : 0);	// any non-zero mark will do
	}
	
	// change the current color	
	::CMGetMenuItemColor(menu_h, item_number, current_color);
	::InvalRect(&window_ptr->portRect);

} // handle_color_menu

// popup menu example
static void do_popup(Point global_pt)
{

	// the menu already has the color strings in it
	MenuHandle		menu_h = ::GetMenu(popup_menu_id);
	if ( !menu_h )
	{
		return;
	}
	::InsertMenu(menu_h, -1);
	
	long	result = ::PopUpMenuSelect(menu_h, global_pt.v + 5, global_pt.h + 5, 0);
	short	item_number = LoWord(result);
	
	if ( result )
	{
		// change the current color	
		::CMGetMenuItemColor(menu_h, item_number, current_color);
		::InvalRect(&window_ptr->portRect);
	}

	::DeleteMenu(popup_menu_id);

} // do_popup
